import pandas as pd
from sklearn.model_selection import train_test_split
from sklearn.metrics import classification_report, confusion_matrix,accuracy_score
from sklearn.feature_extraction.text import CountVectorizer
from sklearn.linear_model import LogisticRegression
from nltk.corpus import stopwords
import nltk
import re
from nltk.stem import PorterStemmer
from nltk.stem import WordNetLemmatizer
import os
import seaborn as sns
import matplotlib.pyplot as plt

# Get the absolute path to the CSV file
file_path = os.path.abspath('data/ER_Analysis_Edited.csv')
# Get the absolute path to the CSV file
file_path_next = os.path.abspath('data/ER_Analysis_Edited.csv')

# Read the CSV file into a DataFrame
df = pd.read_csv(file_path)
df1 = pd.read_csv(file_path_next)

df  = df.applymap(lambda x: x.lower() if isinstance(x, str) else x)


def combine_words(entity):
    return ', '.join(['_'.join(word.split()) for word in entity.split(', ')])

# Apply the combine_words function to the "entity" column
df["Entities"] = df["Entities"].apply(combine_words)


df


import re


# Take each word one by one and append it to the list
entity_list = []
for entity in df['Entities']:
    words = re.findall(r'\b\w+\b', entity)
    entity_list.extend(words)
    

# Print the words
for word in entity_list:
    print(word)



# Take each word one by one and append it to the list
attribute_list = []
for entity in df['Attributes']:
    words = re.findall(r'\b\w+\b', entity)
    attribute_list.extend(words)

# Print the words
for word in attribute_list:
    print(word)



import re

relations = []
for relation in df['Relationship']:
    if isinstance(relation, str): 
        words = re.findall(r'\b\w+\b', relation)
        relations.extend(words)

for word in relations:
    print(word)



df_rel = pd.DataFrame(relations, columns=['Name'])
df_rel['Class']='relation'


df_rel


df_attribute = pd.DataFrame(attribute_list,columns=['Name'])
df_attribute['Class']='attribute'
df_attribute


df_entity = pd.DataFrame(entity_list,columns=['Name'])
df_entity['Class']='entity' 


df_new = pd.concat([df_entity, df_rel], axis=0)


df=df_new


df


# Create a CountVectorizer to convert words into numerical features
vectorizer = CountVectorizer()

# Fit and transform the words in your dataset into numerical features
X = vectorizer.fit_transform(df['Name'])

# Encode labels
from sklearn.preprocessing import LabelEncoder
label_encoder = LabelEncoder()
y = label_encoder.fit_transform(df['Class'])

# Split the dataset into train and test sets
X_train, X_test, y_train, y_test = train_test_split(X, y, test_size=0.2, random_state=42)

# Train a logistic regression classifier
classifier = LogisticRegression(max_iter=1000, random_state=42)
classifier.fit(X_train, y_train)



# Evaluate the classifier on the test set
y_pred = classifier.predict(X_test)
accuracy = accuracy_score(y_test, y_pred)
print(f"Accuracy on test set: {accuracy * 100:.2f}%")

# Print classification report for detailed evaluation
report = classification_report(y_test, y_pred, target_names=label_encoder.classes_)
print(report)


input_word = "order"
preprocessed_word = vectorizer.transform([input_word])
predicted_class = label_encoder.inverse_transform(classifier.predict(preprocessed_word))[0]

# Print the predicted class
print(f"Input word: {input_word}")
print(f"Predicted class: {predicted_class}")


# Initialize NLTK Porter Stemmer and WordNet Lemmatizer
stemmer = PorterStemmer()
lemmatizer = WordNetLemmatizer()

# Define a function to preprocess an input sentence with stop word removal, stemming, and lemmatization
def preprocess_sentence(sentence):
    words = re.findall(r'\b\w+\b', sentence.lower())
    # Remove stop words
    stop_words = set(stopwords.words('english'))
    words = [word for word in words if word not in stop_words]
    lemmatized_words = [lemmatizer.lemmatize(word) for word in words]
    return ' '.join(lemmatized_words)

# Define a function to predict the class of each word in a sentence
def predict_word_classes(sentence):
    words = sentence.split() 
    predicted_classes = []
    
    for word in words:
        preprocessed_word = vectorizer.transform([word])
        predicted_class = label_encoder.inverse_transform(classifier.predict(preprocessed_word))[0]
        predicted_classes.append((word, predicted_class))
    
    return predicted_classes
input_sentence = "A company wants to develop a system to manage their inventory. They have products with attributes such as name, description, price, and quantity in stock. Each product belongs to a specific category. The company wants to track customer orders, including the products ordered, the quantity of each product, and the customer details like name, address, and contact information"

preprocessed_input = preprocess_sentence(input_sentence)
word_classes = predict_word_classes(preprocessed_input)


unique_entities = []
unique_relations = []

# Define a function to extract and store unique entities and relations
def extract_unique_entities_and_relations(sentence):
    words = sentence.split()  # Split the sentence into words
    
    for word in words:
        preprocessed_word = vectorizer.transform([word])
        predicted_class = label_encoder.inverse_transform(classifier.predict(preprocessed_word))[0]
        
        if predicted_class == 'entity' and word not in unique_entities:
            unique_entities.append(word)
        elif predicted_class == 'relation' and word not in unique_relations:
            unique_relations.append(word)

#input_sentence = "Shipped items are the heart of the UPS product tracking information system. Shipped items can be characterized by item number (unique), weight, dimensions, insurance amount, destination, and final delivery date. Shipped items are received into the UPS system at a single retail center. Retail centers are characterized by their type, uniqueID, and address. Shipped items make their way to their destination via one or more standard UPS transportation events (i.e., flights, truck deliveries). These transportation events are characterized by a unique scheduleNumber, a type (e.g, flight, truck), and a deliveryRoute."
preprocessed_input = preprocess_sentence(input_sentence)
extract_unique_entities_and_relations(preprocessed_input)


unique_entities


import re

def extract_unique_entities_and_relation(user_input):
    scenario = f"I need to extract entities and attribute for ER diagram from {user_input}\n Attribute standard should be same for each entity 'first letter_attribute name', example output: ' Entities: \n customer':'c_id, c_name'.\n dont show numbers and Relationships or anything other than the given example in ur output";import openai;openai.api_type = "azure";openai.api_base = "https://openitest.openai.azure.com/";openai.api_version = "2022-12-01";openai.api_key = "c461434794eb43f2ad9be371728f70c4";model_engine = "text-davinci-003";response = openai.Completion.create(engine=model_engine,prompt=scenario,temperature=0,max_tokens=3500,top_p=1,frequency_penalty=0,presence_penalty=0,best_of=1,stop=None);output_text = response.choices[0].text.strip();return output_text


att_list = df1['Attributes'].tolist()


att_list = list(set(att_list))


att_list=str(att_list)


import pandas as pd
def extract_cardinality_and_relationship(user_input):
    scenario="please identify 'Entities for ER diagram and extract relationship(should be one single verb) between entities seperatly from "+user_input+" output should be seperate sentences. I dont need the entity list in your output. use entities in "+output_ent_attr+" Output format must be 'Entity---Relation---Entity---Cardinality(one-to-one/one-to-many/many-to-many)' between all entities";import openai;openai.api_type = "azure";openai.api_base = "https://openitest.openai.azure.com/";openai.api_version = "2022-12-01";openai.api_key = "c461434794eb43f2ad9be371728f70c4";prompt = scenario;model_engine="text-davinci-003";response = openai.Completion.create(engine=model_engine,prompt=prompt,temperature=0,max_tokens=3500,top_p=1,frequency_penalty=0,presence_penalty=0,best_of=1,stop=None);output_text = response.choices[0].text.strip();return output_text      
    


att_list


allowed_characters = set('_,')

# Remove all punctuation except allowed characters
cleaned_string = ''.join(char for char in att_list if char.isalnum() or char in allowed_characters)

# Output the cleaned string
print(cleaned_string)


output_list = cleaned_string.split(',')
print(output_list)


output_list = list(set(output_list))


def generate_entitiy_attributes(sentence, variable_name, attributes):

    sentence_words = re.findall(r'\b\w+\b', sentence.lower())
    # Convert the variable_name and attributes to lowercase 
    variable_name = variable_name.lower()
    attributes = [attr.lower() for attr in attributes]

    if variable_name in sentence_words:
        # Check which attributes are present 
        attributes_present = [attr for attr in attributes if attr in sentence_words]
        return {"entity": variable_name, "attributes": attributes_present, "confirmed": True}
    else:
        return {"entity": variable_name, "attributes": [], "confirmed": False}

#att_list = ["prod_id", "prod_name", "prod_description", "prod_price", "prod_quantity", "cat_id", "cat_name", "order_id", "order_date", "cus_id", "cus_name", "cus_address", "cus_contact"]
attributes_list = output_list

entity_list = unique_entities
print()
sentences="A virtual supermarket has registered suppliers to supply the customer orders placed online. The supermarket always fulfils its customer orders through these suppliers. One supplier is responsible only for the customers who live in the supplier's area. A customer has only one supplier. Each supplier is characterized by a code (unique), address and contact numbers. A supplier can have several contact numbers. Each customer is characterized by an email address (unique), name and location. A customer can confirm orders. Each order has only one supplier and one customer. An order is characterized by an order number (unique), description and a value. A supplier can supply more than one order."

confirmed_entities = [] 
for entity in entity_list:
    entity_result = [generate_entitiy_attributes(sentence, entity, attributes_list) for sentence in sentences]
    confirmed_result = [result for result in entity_result if result["confirmed"]]
    confirmed_entities.extend(confirmed_result)

import sys

if __name__ == "__main__":
    user_input = sys.argv[1]

# user_input = input("Please enter your input: ")
user_input= user_input.replace('"', '')

output_ent_attr = extract_unique_entities_and_relation(user_input)
print(output_ent_attr)

lines = output_ent_attr.split('\n')
# dictionary to store attributes for each entity
attributes = {}
if lines[0].strip().lower() == "entities:":
    lines = lines[1:]

for line in lines:
    parts = line.split(':')
    entity_name = parts[0].strip()
    entity_attributes = [attr.strip() for attr in parts[1].split(',')]

    # If the entity does not have a primary key, generate one
    if not any(attr.endswith('ID') or attr.endswith('id') or attr.endswith('Id') for attr in entity_attributes):
        primary_key = f"{entity_name.lower()[:1]}_id"
        entity_attributes.insert(0, primary_key)

    attributes[entity_name] = entity_attributes

entity_list = list(attributes.keys())

print("entity_list =", entity_list)
print("attributes =", attributes)



import re

def extract_cardinelity_and_relationship(sentence):
    
    one_to_one_pattern = re.compile(r'(\bone\b|\b1\b) (\w+) has (\bone\b|\b1\b) (\w+)')
    one_to_many_pattern = re.compile(r'(\b1\b) (\w+) has (\bmany\b|\bmultiple\b) (\w+)')

    # Search for patterns in the sentence
    match_one_to_one = one_to_one_pattern.search(sentence)
    match_one_to_many = one_to_many_pattern.search(sentence)

    if match_one_to_one:
        entity1, relationship, entity2 = match_one_to_one.group(2, 3, 4)
        return f" {entity1} ---{relationship} ---{entity2}"
    elif match_one_to_many:
        entity1, relationship, entity2 = match_one_to_many.group(2, 3, 4)
        return f" {entity1} --- {relationship} --- {entity2}"
    else:
        return "No cardinality or relationship pattern found."





relation_cardinality=extract_cardinality_and_relationship(user_input)
print(relation_cardinality)



relations = {}

lines = relation_cardinality.strip().split('\n')

for line in lines:
    left_entity, relationship, right_entity, cardinality = line.split('---')
    left_cardinality, right_cardinality = cardinality.split('-to-')

    left_cardinality = left_cardinality.replace("One", "1").replace("Many", "M")
    right_cardinality = right_cardinality.replace("One", "1").replace("Many", "M")

    relations[relationship] = {
        "left": (left_entity, left_cardinality),
        "right": (right_entity, right_cardinality)
    }

print(relations)



import networkx as nx
import matplotlib.pyplot as plt

def draw_er_diagram(entity_list, attributes, relations):
    G = nx.Graph()

    # Add entities as nodes with square shape
    for entity in entity_list:
        G.add_node(entity, shape='s')

    # Add attributes as nodes with circle shape
    for entity, attrs in attributes.items():
        for attr in attrs:
            attr_with_entity = f"{attr}"
            G.add_node(attr_with_entity, shape='o')
            G.add_edge(entity, attr_with_entity)

    # Add relations as nodes with diamond shape
    for relation, sides in relations.items():
        G.add_node(relation, shape='d')
        left_entity, left_cardinality = sides["left"]
        right_entity, right_cardinality = sides["right"]
        G.add_edge(left_entity, relation)
        G.add_edge(relation, right_entity)

    pos = nx.spring_layout(G, seed=62, k=0.2)
  
    entity_marker = 's'
    attribute_marker = 'o'
    relationship_marker = 'd'

    entity_nodes = [node for node in G.nodes if node in entity_list]
    attribute_nodes = [node for node in G.nodes if node not in entity_list]

    plt.figure(figsize=(12, 8))
    nx.draw_networkx_nodes(G, pos, nodelist=entity_nodes, node_shape=entity_marker, node_color='skyblue',
                           node_size=1000, alpha=0.9, linewidths=0.5)
    nx.draw_networkx_nodes(G, pos, nodelist=attribute_nodes, node_shape=attribute_marker, node_color='lightgreen',
                           node_size=800, alpha=0.9, linewidths=0.5)
    nx.draw_networkx_nodes(G, pos, nodelist=list(relations.keys()), node_shape=relationship_marker, node_color='gray',
                           node_size=1200, alpha=0.9, linewidths=0.5)
    nx.draw_networkx_edges(G, pos, edge_color='gray', alpha=0.5, width=0.5)
    nx.draw_networkx_labels(G, pos, font_color='black', font_size=8)
    
    # Adding cardinality labels to the edges
    cardinality_labels = {}
    for relation, sides in relations.items():
        left_entity, left_cardinality = sides["left"]
        right_entity, right_cardinality = sides["right"]
    
        cardinality_labels[(left_entity, relation)] = left_cardinality
        cardinality_labels[(relation, right_entity)] = right_cardinality
    
    pos_labels = {k: (v[0], v[1] + 0.03) for k, v in pos.items()}
    nx.draw_networkx_edge_labels(G, pos_labels, edge_labels=cardinality_labels, font_size=8, font_color='red')

    #plt.title("Entity-Relationship Diagram")
    diagram_path = os.path.abspath('static/er_diagram.png')
    plt.axis('off')
    plt.savefig(diagram_path, format="png")
    # plt.show()


draw_er_diagram(entity_list, attributes, relations)



print("entity_list =", entity_list)
print("attributes =", attributes)
print("relations=",relations)

# SQL statements to create tables for the relations
table_creation_statements = []

for entity in entity_list:
    attributes_list = attributes[entity]
    attribute_str = ', '.join([f'{attribute} VARCHAR(255)' for attribute in attributes_list])
    table_creation_query = f"CREATE TABLE {entity} ({attribute_str});"
    table_creation_statements.append(table_creation_query)

# SQL statements to create foreign keys for the relations
foreign_key_statements = []

for relation, details in relations.items():
    left_entity, left_cardinality = details["left"]
    right_entity, right_cardinality = details["right"]
    
    foreign_key_query = f"ALTER TABLE {left_entity} " \
                        f"ADD CONSTRAINT FK_{left_entity}_{right_entity} " \
                        f"FOREIGN KEY ({left_entity.lower()}_id) " \
                        f"REFERENCES {right_entity}({right_entity.lower()}_id);"
    
    foreign_key_statements.append(foreign_key_query)

import json

# Print only the SQL statements for Flask to use
if __name__ == "__main__":
    sql_queries = "\n".join(table_creation_statements + foreign_key_statements)
    
    # Split the SQL queries into lines
    queries_lines = sql_queries.split('\n')
    
    # Filter relevant lines starting with "CREATE TABLE" or "ALTER TABLE"
    relevant_queries = [line for line in queries_lines if line.startswith(('CREATE TABLE', 'ALTER TABLE'))]
    
    # Join the relevant queries into a string
    formatted_queries = "\n".join(relevant_queries)
    print(formatted_queries)

    query_path = os.path.abspath('data/sql_queries.json')
    # Save formatted_queries to a JSON file (optional)
    with open(query_path, 'w') as sql_file:
        json.dump(formatted_queries, sql_file)


entity_path = os.path.abspath('data/entity_list.json')
# Save entity_list to a JSON file
with open(entity_path, 'w') as entity_file:
    json.dump(entity_list, entity_file)

attribute_path = os.path.abspath('data/attributes.json')
# Save attributes to a JSON file
with open(attribute_path, 'w') as attributes_file:
    json.dump(attributes, attributes_file)

relationship_path = os.path.abspath('data/relations.json')
# Save relations to a JSON file
with open(relationship_path, 'w') as relations_file:
    json.dump(relations, relations_file)

print("JSON files saved successfully.")